      SUBROUTINE FILTER(NDP,X,KPOL,ICLR,AN,AD,W)
C*ABSTRACT
C
C      DIGITAL RECURSIVE FILTER.
C      APPLICATION SECTION OF COBFILT
C
C*ABSTRACT
C
C
C   JRC   2 - 12 - 88  EXTRACTED FROM COBFILT
C   V. W. MOORE  012/22/77
C
C   THIS SUBROUTINE MAY BE USED TO APPLY EITHER A
C   DIGITAL RECURSIVE FILTER DESIGNED BY MAKFIL
C   TO FLOATING-POINT TIME-SERIES DATA.
C
C
C      INPUT PARAMETERS:
C
C            NDP IS THE NUMBER OF DATA POINTS.
C
C            X IS THE FLOATING-POINT INPUT/OUTPUT ARRAY.
C
C            ICLR IS A CONTROL PARAMETER. 
C                  CLEARS THE STORAGE ARRAYS IF .NE. 0
C
C            KPOL IS THE NUMBER OF COEFFICIENTS.
C
C            AN AND AD ARE THE FILTER COEFFICIENT ARRAYS
C               COMPUTED WHEN LOM=1 AND USED IN SUBSEQUENT
C               FILTERING.  DIMENSIONS SHOULD BE AT LEAST
C               K+3 FOR LOWPASS AND 2*K+1 FOR BANDPASS.
C            W IS AN ARRAY USED  FOR FILTER FEEDBACK FROM
C               ONE CALL TO THE NEXT. CLEARED IF ICLR .NE. 0
C               DIMENSIONS SHOULD BE AT LEAST K+1 FOR
C               LOWPASS AND 2*K+1 FOR BANDPASS.
C
C   FORTRAN LIBRARY ROUTINES CALLED:
C      SIN
C      COS
C      SQRT
C
C   REFERENCE:
C      RADER AND GOLD,"DIGITAL FILTER DESIGN
C       TECHNIQUES IN THE FREQUENCY DOMAIN",
C       PROC. IEEE V.55 PP.149-171 (1967).
C 
C   =========  DOUBLE PRECISION ALL F. P.  ==============
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION      X(*), AN(3), AD(3), W(*)
C 
C___THIS VALUE OF PI WILL BE PROPERLY INTERPRETED AS IT APPEARS HERE.
C   THIS METHOD OF ENTERING DATA IN A DATA STATEMENT IS SUPPORTED
C   BY THE FORTRAN 77 STANDARD.  THE BLANKS ARE NOT ZEROES.
C   USERS MAY INCLUDE THIS IN THEIR MODIFY LIBRARIES.
C
      DATA    PI/ 3.14159 26535 89793 23846 26433 83279 50288 41972 /
C
C
C-----------------------------------------------------------------------
C
C  PERFORM FILTER OPERATION (TERMINATE IF NO FILTER HAS BEEN DESIGNED)
C
200   CONTINUE
C
      M = KPOL-1
C
C  CONTINUE FILTER USING FEEDBACK FROM LAST CALL IF ICLR .EQ. 0
C     OTHERWISE, CLEAR FEEDBACK ARRAY
C
      IF (ICLR.EQ.0 ) GO TO 220
C
      DO 210 I=1,M
         W(I) = 0.0D0
 210  CONTINUE
C
C  MAIN FILTER LOOP - EXECUTE ONCE FOR EACH INPUT POINT
C
 220  CONTINUE
C
      DO 260 I=1,NDP
C
         W(KPOL) = X(I)
C
C  COMPUTE DIFFERENCE EQUATIONS
C
         DO 230 J=1,M
            W(KPOL) = W(KPOL)-AD(J)*W(J)
 230     CONTINUE
C
         Q = W(KPOL)*AN(KPOL)
         DO 240 J=1,M
            Q = Q+AN(J)*W(J)
 240     CONTINUE
         X(I) = Q
C
C  SHIFT FEEDBACK ARRAY ONE POSITION
C
         DO 250 J=1,M
            W(J) = W(J+1)
 250     CONTINUE
 260  CONTINUE
C
C
900   CONTINUE
C
      RETURN
      END
