$TITLE:'KALMAN TRACKERS'
$SUBTITLE:'INIT X-XDOT TRACKER'
        SUBROUTINE IKXVT ( TZ,XZ,VZ,SXZ,SVZ,QRATEX,QRATEV,
     $                     XSTATE,XKWORK )
C
C          INITIALIZE KALMAN TWO STATE TRACKER
C
C          INPUT MEASUREMENT ... IE X
C          TRACK   X, XDOT ( X AND V )
C
C
C         INPUT:
C
C               INITIAL STATE 
C
C                   TZ        R*8       TIME OR INDEPENDENT VAR
C                   XZ        R*8       MEASUREMENT OF X
C                   SXZ       R*8       SIGMA  X
C                   SVZ       R*8       SIGMA  V ( XDOT )
C
C                PROCESS NOISE RATES
C
C                   QRATEX    R*8       RATE OF X NOISE (SEE BELOW)
C                   QRATEV    R*8       RATE OF V NOISE
C
C         OUTPUT
C 
C                   XSTATE ARRAY 3 LONG ( NOTE 3 )
C
C                   XSTATE(1)      R*8  X ESTIMATE
C                   XSTATE(2)      R*8  V ( XDOT ) ESTIMATE
C                   XSTATE(3)      R*8  T -- TIME OF ESTIMATE
C
C
C                   WORK ARRAY 5 LONG
C
C                   XKWORK (1)     R*8  P(11) = SIG**2 X
C                   XKWORK (2)     R*8  P(22) = SIG**2 V
C                   XKWORK (3)     R*8  P(23) = SIG**2 XV
C                                       ( P SYMETRIC SO ONLY HALF)
C                   XKWORK (4)     R*8  qX -- TIME RATE PROC NOISE X
C                   XKWORK (5)     R*8  qV    "   V
C
C
C    NOTES:
C
C
C           ASSUMES IS A DETERMISTIC HIGHER ORDER DERIVATIVE
C           THAT DRIVES PROCESS NOISE.  --  THEREFORE USED
C           PROCESS NOISE MODEL BASED ON TIME SINCE LAST DATA
C
C            QXX   =    ( qX * DT )**2
C            QVV   =    ( qV * DT )**2
C            QXV    =      0.0
C
C           ( NO DIGNIAL COEFICIENTS )
C
C            qX     SHOULD BE  A * DT 
C            qV     SHOULD BE  A
C                   WHERE  "A" IS THE MAGNITUDE OF ACCELERATION
C                   ( PLUS SOME FOR RANDOM EFFECTS )
C
$INCLUDE:'WRDSIZ.INC'
C
C      =============    SPECIFICATIONS   =================
C
       DIMENSION   XSTATE(3), XKWORK(5)
C
C      ===================================================
C
C ---------------------------------------------------------------------
C
      XSTATE(1)   =   XZ
      XSTATE(2)   =   VZ
      XSTATE(3)   =   TZ
C
      XKWORK(1)   =   SXZ**2
      XKWORK(2)   =   SVZ**2
      XKWORK(3)   =   0.0D0
      XKWORK(4)   =   DABS ( QRATEX )
      XKWORK(5)   =   DABS ( QRATEV )
C
C
      RETURN
      END
$PAGE
$SUBTITLE:'X-XDOT TRACKER'
        SUBROUTINE KXVTK ( T, XM, SIGM, XSTATE,XKWORK )
C
C          KALMAN TWO STATE TRACKER
C
C          INPUT MEASUREMENT ... IE X
C          TRACK   X, XDOT ( X AND V )
C
C
C         INPUT:
C                   T    R*8       TIME OR INDEPENDENT VAR
C                   XM   R*8       MEASUREMENT OF X
C                   SIGM R*8       SIGMA  OF MEASUREMENT
C
C         INPUT/OUTPUT
C 
C                   XSTATE ARRAY 3 LONG ( NOTE 3 )
C
C                   XSTATE(1)      R*8  X ESTIMATE
C                   XSTATE(2)      R*8  V ( XDOT ) ESTIMATE
C                   XSTATE(3)      R*8  T -- TIME OF ESTIMATE
C
C          INPUT/OUTPUT/WORK
C
C                   WORK ARRAY 5 LONG
C
C                   XKWORK (1)     R*8  P(11) = SIG**2 X
C                   XKWORK (2)     R*8  P(22) = SIG**2 V
C                   XKWORK (3)     R*8  P(23) = SIG**2 XV
C                                       ( P SYMETRIC SO ONLY HALF)
C                   XKWORK (4)     R*8  qX -- TIME RATE PROC NOISE X
C                   XKWORK (5)     R*8  qV    "   V
C
C
C         EFFECTIVE MODEL
C                                         
C                   X      =   [X ]
C                              [XDOT]
C
C                   H      =   [ 1,0 ]
C
C                   PHI    =   [ 1, DT  ]
C                              [ 0,  1  ]
C
C                   Q      =   [ QXX ,  0   ]
C                              [   0 ,  QVV ]
C
C
C
C     NOTES:
C
C       1.  USE  IKXVT  TO INITIALIZE
C
C       2.  ASSUMES IS A DETERMISTIC HIGHER ORDER DERIVATIVE
C           THAT DRIVES PROCESS NOISE.  --  THEREFORE USED
C           PROCESS NOISE MODEL BASED ON TIME SINCE LAST DATA
C
C            QXX   =    ( qX * DT )**2
C            QVV   =    ( qV * DT )**2
C            QXV    =      0.0
C
C           ( NO DIGNIAL COEFICIENTS )
C
C            qX     SHOULD BE  A * DT 
C            qV     SHOULD BE  A
C                   WHERE  "A" IS THE MAGNITUDE OF ACCELERATION
C                   ( PLUS SOME FOR RANDOM EFFECTS )
C
C       3.  HAS ALL STOAGE IN VECTORS SO CAN USE FOR SEVERAL 
C           TRACKERS AT ONCE
C
C       4.  SAVES TIME AS PART OF STATE -- ALLOW FOR SPACE
C
C       5.  DOES ALL MATH AS SCALERS.  SMALL ENOUGH CAN DO 
C           MATRIX ITEMS ONE ELEMENT AT A TIME WITH ANALYTIC
C           EXPRESSIONS FOR COMPONENTS
C
$INCLUDE:'WRDSIZ.INC'
C
C      =============    SPECIFICATIONS   =================
C
       DIMENSION   XSTATE(3), XKWORK(5)
C
C      ===================================================
C
C ---------------------------------------------------------------------
C
C           ----------------------------------------------
C            PROPAGATE  THE STATE AND CO-VARIANCE MATRIX
C            -------------------------------------------
C
C
C       -----  TIME AND TIME DIFERENCE ----
C
       DT    =  T - XSTATE (3)
       DTA   =  ABS ( DT )
C
C          ----   STATE PROPAGATE  ----
C
       XNOW  =  XSTATE(1)
       VNOW  =  XSTATE(2)
C
       XP    =  XNOW  +  DT * VNOW
       VP    =  VNOW
C
C         ---  COVARIANCE PROPAGATE  ---
C
       VARX  =  XKWORK(1)
       VARV  =  XKWORK(2)
       VARXV =  XKWORK(3)
C
       QRATEX =  XKWORK(4)
       QRATEV =  XKWORK(5)
C
       QXX    =  ( QRATEX*DT) ** 2
       QVV    =  ( QRATEV*DT) ** 2
C
       VARXP  =  VARX  +  2.0D0*VARXV*DTA  +  VARV*DTA*DTA
       VARVP  =  VARV
       VARXVP =  VARXV +  VARV*DTA
C
       VARXP  =  VARXP + QXX
       VARVP  =  VARVP + QVV
C
C        ------------------------------------
C          KALMAN GAIN  G = TRANS  [ GX, GV ]
C        ------------------------------------
C
C                HPHT   =  JUST 11 COMPONENT OF PROP(P)  ==  P(+)
C                DOMINATOR OF GAIN IS THAT + MEASURMENT NOISE
C
C                       P(+) * HT   JUST SELECTS FIRST COLUMN
C                                    OF P(+)      
C
       DOM    =  VARXP + SIGM**2
C
       IF ( DABS ( DOM ) .GT. 1.0D-40 ) THEN
          GX      =   VARXP  / DOM
          GV      =   VARXVP / DOM
       ELSE
          GX       = 0.0D0
          GV       = 0.0D0
       ENDIF
C
C
C          -----------------------------
C          RESIDUAL AND UPDATE OF STATE 
C          -----------------------------
C
      RES  =  XM -  XP
C
      XNEW  =  XP  +  GX * RES
      VNEW  =  VP  +  GV * RES
C
C
C       ----------------------------------
C          UPDATE CO-VARIANCE
C       ----------------------------------
C
      VARXN  = ( 1.0D0 -  GX )  *  VARXP
      VARVN  =   -GV * VARXVP   +  VARVP
      VARXVN = ( 1.0D0 -  GX )  *  VARXVP
C
C
C  -------------   STORE RESULTS IN VECTORS  ------------
C
      XSTATE(1)   = XNEW
      XSTATE(2)   = VNEW
      XSTATE(3)   = T
C
      XKWORK(1)   =  VARXN
      XKWORK(2)   =  VARVN
      XKWORK(3)   =  VARXVN
C
C
C         ----------  EXIT   ---------
C
      RETURN
      END
