$TITLE:'MAGNETIC CO-ORDINATES'
$SUBTITLE:'DIPOLE, GC LAT IN'
      SUBROUTINE GC2MDP ( GCLAT,GLON,FMLAT,FMLON )
C
C         GEOCENTRIC TO DIPOLE MAGNETIC CO-ORDINATES
C
C              AIP HANDBOOK 5-265 DIRECT EQUATIONS
C                   PHIZERO = 90 - 11.44 DEG
C                   LAMDAZ  = 290.24     DEG
C
C    INPUT:
C              GCLAT          R*8       GEOCENTRIC LAT
C              GLON           R*8       GEOGRAPHIC LON
C
C    OUTPUT:
C              FMLAT          R*8       MAG. DIPOLE LAT
C              FMLON          R*8       MAG. DIPOLE LON
C
C
C    LASTMOD:
C              JRC  3-4-95   CODED
C
C
$INCLUDE:'WRDSIZ.INC'
C
      DATA   DTR /   1.7453 292519943D-2 /
      DATA   RTD /  57.29577 951308 2 D0 /
C
      DATA   CLATZ/  11.44D0 /
      DATA   FLZ  / 290.24D0 /
C
      DATA    INIT  / 0 /
C
C
C         -----------------------------------------
C
       IF ( INIT.EQ. 0 ) THEN
           SCLZ  =  SIN ( DTR * CLATZ )
           CCLZ  =  COS ( DTR * CLATZ )
           INIT  =  1
       ENDIF
C
C
       XLAT  = 0.0D0
       XLON  = 0.0D0
C
       IF ( GCLAT.GT.90.0D0  .OR.  GCLAT.LT. -90.0D0 ) GOTO 900
C
       COLAT = DTR * ( 90.0D0 - GCLAT )
       DLON  = DTR * ( GLON - FLZ  )
C
       SCL    = SIN ( COLAT )
       CCL    = COS ( COLAT )
       SDL    = SIN ( DLON )
       CDL    = COS ( DLON )
C
C              COS MAGNETIC CO-LATITUDE
C
       CCLM  = CCL*CCLZ + SCL*SCLZ*CDL
C
       IF ( CCLM.GT. 1.0D0 ) CCLM =  1.0D0
       IF ( CCLM.LT.-1.0D0 ) CCLM = -1.0D0
C
C         FIND NEW UNIT VECTOR XM,YM FOR MAG LONG.
C
       XM =  CCLZ*SCL*CDL - SCLZ*CCL
       YM =  SCL*SDL 
C
       XCLAT =  RTD * ACOS ( CCLM )
       XLAT  =  90.0D0  - XCLAT
       XLON  =  RTD * ATAN2 ( YM,XM )
C
       IF ( XLON.LT.0.0D0 )   XLON = XLON + 360.0D0
C
900   CONTINUE
C
       FMLAT  = XLAT
       FMLON  = XLON
C
        RETURN
        END
$PAGE
$SUBTITLE:'GEODETIC TO MAG DIPOLE'
      SUBROUTINE GD2MDP ( GDLAT,GLON,FMLAT,FMLON )
C
C         GEOCENTRIC TO DIPOLE MAGNETIC CO-ORDINATES
C
C              AIP HANDBOOK 5-265 DIRECT EQUATIONS
C                   PHIZERO = 90 - 11.44 DEG
C                   LAMDAZ  = 290.24     DEG
C
C    INPUT:
C              GDLAT          R*8       GEODETIC   LAT
C              GLON           R*8       GEOGRAPHIC LON
C
C    OUTPUT:
C              FMLAT          R*8       MAG. DIPOLE LAT
C              FMLON          R*8       MAG. DIPOLE LON
C
C
C    LASTMOD:
C              JRC  3-4-95   CODED
C
C
$INCLUDE:'WRDSIZ.INC'
C
      DATA   DTR /   1.7453 292519943D-2 /
      DATA   RTD /  57.29577 951308 2 D0 /
C
      DATA   CLATZ/  11.44D0 /
      DATA   FLZ  / 290.24D0 /
C
      DATA    INIT  / 0 /
C
C
C         -----------------------------------------
C
       IF ( INIT.EQ. 0 ) THEN
           SCLZ  =  SIN ( DTR * CLATZ )
           CCLZ  =  COS ( DTR * CLATZ )
           INIT  =  1
       ENDIF
C
       ALTKM = 0.0D0
       CALL GD2GC ( GDLAT,ALTKM,GCLAT )
C
C
       XLAT  = 0.0D0
       XLON  = 0.0D0
C
       IF ( GCLAT.GT.90.0D0  .OR.  GCLAT.LT. -90.0D0 ) GOTO 900
C
       COLAT = DTR * ( 90.0D0 - GCLAT )
       DLON  = DTR * ( GLON - FLZ  )
C
       SCL    = SIN ( COLAT )
       CCL    = COS ( COLAT )
       SDL    = SIN ( DLON )
       CDL    = COS ( DLON )
C
C              COS MAGNETIC CO-LATITUDE
C
       CCLM  = CCL*CCLZ + SCL*SCLZ*CDL
C
       IF ( CCLM.GT. 1.0D0 ) CCLM =  1.0D0
       IF ( CCLM.LT.-1.0D0 ) CCLM = -1.0D0
C
C         FIND NEW UNIT VECTOR XM,YM FOR MAG LONG.
C
       XM =  CCLZ*SCL*CDL - SCLZ*CCL
       YM =  SCL*SDL 
C
       XCLAT =  RTD * ACOS ( CCLM )
       XLAT  =  90.0D0  - XCLAT
       XLON  =  RTD * ATAN2 ( YM,XM )
C
       IF ( XLON.LT.0.0D0 )   XLON = XLON + 360.0D0
C
900   CONTINUE
C
       FMLAT  = XLAT
       FMLON  = XLON
C
        RETURN
        END
$PAGE
$SUBTITLE:'ECCENTRIC DIPOLE/GEODETIC IN'
      SUBROUTINE GD2MED ( GDLAT,GLON,FMLAT,FMLON )
C
C         GEOCENTRIC TO ECCENTRIC DIPOLE MAGNETIC CO-ORDINATES
C
C              HEARN ET AL    JGR (92? ) 1666
C
C
C    INPUT:
C              GDLAT          R*8       GEODETIC   LAT
C              GLON           R*8       GEOGRAPHIC LON
C
C    OUTPUT:
C              FMLAT          R*8       MAG. DIPOLE LAT
C              FMLON          R*8       MAG. DIPOLE LON
C
C
C    LASTMOD:
C              JRC  3-4-95   CODED
C
C
$INCLUDE:'WRDSIZ.INC'
C
      DIMENSION    XE(3) ,XDIF(3), XEDP(3), XZERO(3), ROT(3,3)
C
      DATA   DTR /   1.7453 292519943D-2 /
      DATA   RTD /  57.29577 951308 2 D0 /
C
      DATA   CLATZ/  11.06710D0 /
      DATA   FLZ  / 288.97910D0 /
C
      DATA   XZERO / -392.628,  258.353,  180.555 /
C
      DATA    INIT  / 0 /
C
C
C         -----------------------------------------
C
       IF ( INIT.EQ. 0 ) THEN
C
           SCLZ  =  SIN ( DTR * CLATZ )
           CCLZ  =  COS ( DTR * CLATZ )
           SLZ   =  SIN ( DTR * FLZ )
           CLZ   =  COS ( DTR * FLZ )
C
           ROT (1,1)  =    CCLZ*CLZ
           ROT (1,2)  =    CCLZ*SLZ
           ROT (1,3)  =   -SCLZ
C
           ROT (2,1)  =   -SLZ
           ROT (2,2)  =    CLZ
           ROT (2,3)  =    0.0D0
C
           ROT (3,1)  =   SCLZ*CLZ
           ROT (3,2)  =   SCLZ*SLZ
           ROT (3,3)  =   CCLZ
C
           INIT  =  1
C
       ENDIF
C
C                   -----------------------------
C
      ALTKM  = 0.0D0
      CALL LLHXYZ ( GDLAT, GLON, ALTKM, XE )
C
      XDIF(1)  =  XE(1) - XZERO(1)
      XDIF(2)  =  XE(2) - XZERO(2)
      XDIF(3)  =  XE(3) - XZERO(3)
C
      CALL  MXV33 ( ROT, XDIF, XEDP )
C
C         USE STRAIGHT SPHERICAL POLAR TO "GEOCENTRIC" COORDINATES
C
      CALL  XYZTSPH ( XEDP, R, FLATM, FLONM )
C
      IF ( FLONM.LT.0.0D0  ) FLONM = FLONM + 360.0D0
      IF ( FLONM.GT.360.0D0) FLONM = FLONM - 360.0D0
C
      FMLAT = FLATM
      FMLON = FLONM
C
900   CONTINUE
      RETURN
      END
