$TITLE:'SOLAR/SUBSOLAR POSITIONS'
$SUBTITLE:'SUB-SOLAR LAT,LON'
      SUBROUTINE SSOLLL ( IYEAR,IDOY, SUNLAT,SUNLON )
C
C         FIND THE SUB SOLAR LATITUDE AND LONGITUDE
C         AT 00 UT ON GIVEN DAY
C
C              Meeus, _Astronomical Formulae for Calculators_
C              Ch. 18, pg79
C
C    INPUT:
C              IYEAR          I*4       YEAR, 2 OR 4 DIGIT
C              IDOY           I*4       DAY OF YEAR
C
C    OUTPUT:
C              SUNLAT         R*8       SUN LATITUDE
C              SUNLON         R*8       SUN  LONGITUDE 
C
$INCLUDE:'WRDSIZ.INC'
C
C         -------------------------------------
C
C              FIND RIGHT ASCENSION AND DEC,  
C              SUBTRACT R.A.  GREENWICH FOR LONG, 
C              DEC  = LATITUDE
C
      CALL SSOLAD ( IYEAR,IDOY, RA, DEC )
C
      DOY  = IDOY  
      YEAR = IYEAR
      TIME = 0.0D0
C
      CALL GHA ( DOY,TIME,YEAR, RAGM )
C
      SLON = RA - RAGM
      IF ( SLON.LT.  0.0D0 ) SLON = SLON + 360.0D0
      IF ( SLON.GT.360.0D0 ) SLON = SLON - 360.0D0
C
      SUNLAT = DEC
      SUNLON = SLON
C
      RETURN
      END
$PAGE
$SUBTITLE:'SUBSOLAR R.A., DECLINATION'
      SUBROUTINE SSOLAD ( IYEAR,IDOY, RA, DEC )
C
C         FIND THE SUBSOLAR  RIGHT ASSENTION AND DECLINATION
C         AT 00 UT ON GIVEN DAY
C
C              Meeus, _Astronomical Formulae for Calculators_
C              Ch. 18, pg79
C
C    INPUT:
C              IYEAR          I*4       YEAR, 2 OR 4 DIGIT
C              IDOY           I*4       DAY OF YEAR
C
C    OUTPUT:
C              RA             R*8       RIGHT ASSENSION  DEG
C              DEC            R*8       DECLINATION DEG
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
C
      DATA   DTR  /  1.7453 292519943D-2   /
      DATA   RTD  / 57.29577 951308 2 D0   /
C
C
C              ---------------------------------------
C
C              COMPUTE JULIAN CENTURIES SINCE JAN 0.5 1990
C                   (JAN 0.5 1990 = 2415020.0 JD )
C                   ( SUBTRACT 2400 000.5 FOR MJD = 15019.5 )
C
      CALL YD2MJD ( IYEAR, IDOY, MJD )
      FMJD    =  MJD
      DDAY    =  FMJD - 15019.5D0
      T       =  DDAY / 36525.0D0
      T2      =  T*T
      T3      =  T*T2
C
C         SUNLON   LONGITUDE OF SUN                   DEG
C         SUNM     MEAN ANOMANLIE OF SUN               "
C         EEO      ECCENTRICITY OF EARTH ORBIT         "
C         EOBQ     EARTH OBLIQUITY                     "
C
      SUNLON = 279.69668D0 + 36000.76892D0*T + 3.025D-4*T2
      ISLCYC = SUNLON/360.0D0
      SUNLON = SUNLON - ISLCYC * 360.0D0
C
      SUNM   = -1.52417D0 + 35999.04975D0*T - 1.5D-4*T2 - 
     $          3.3D-6*T3
      ISMCYC =  SUNM/360.0D0
      SUNM   =  SUNM - ISMCYC * 360.0D0
C
      EEO    =  0.01675104 D0 - 4.18D-5*T - 1.26D-7*T2
      EOBQ   =  23.452294D0   - 1.30125D-2*T  -
     $          1.64D-6*T2  + 5.03D-7*T3
C
      EOBQR  = DTR * EOBQ
C
C         EXPANSION IN M OF CORRECTION TO E
C
      RADM   =  DTR * SUNM
      SM     =  SIN (RADM)
      S2M    =  SIN (2.0D0*RADM)
      S3M    =  SIN (3.0D0*RADM)
C
      C  = ( 1.919460D0 - 4.789D-3*T - 1.4D-5*T2 )*SM +
     $     ( 0.020094D0 - 1.00D-4*T  ) * S2M +
     $     ( 0.000293D0 ) * S3M
C
C         SUN TRUE LONG  = STLON
C
      STLON  = SUNLON + C
      STLONR = DTR * STLON
C
C              NOW CONPUTE R.A. AND DEC.
C
      STL   =  SIN ( STLONR )
      CTL   =  COS ( STLONR )
      SOBQ  =  SIN ( EOBQR )
      COBQ  =  COS ( EOBQR )
C
      SEFF  =  COBQ * STL
      RASUN =  RTD * ATAN2 ( SEFF, CTL )
C
      DECSUN =  RTD * ASIN ( SOBQ * STL )
C
900   CONTINUE
C
      IF ( RASUN.LT.0.0D0 ) RASUN = RASUN + 360.0D0
C
      RA  =  RASUN
      DEC =  DECSUN
C
      RETURN
      END
