$TITLE:'UTM ROUTINES'
$SUBTITLE:'GEODETIC TO UTM'
      SUBROUTINE GD2UTM ( FLAT,FLON, X,Y, SF, IZONE )
C
C           CONVERT LAT, LON INTO UNIVERSAL TRANVERSE MERCATOR
C
C           ( DOES'NT DO STEROGRAPHIC FOR POLAR REGIONS )
C            USES GENERAL TRANSVERS MERCATOR OF NOAS REPORT
C            LISTED BELOW
C
C      INPUT:
C              FLAT     R*8      LATITUDE IN DEG
C              FLON     R*8      LONGITUDE IN DEG
C      OUTPUT:
C              X        R*8      X OR EASTING (LONGITUDE LIKE) 
C              Y        R*8      Y OR NORTHING ( BOTH IN M )
C              SF       R*8      TRUE SCALE FACTOR AT POINT
C              IZONE    I*4      ZONE NUMBER ( 1-60 )
C                                ** MAKE NEGATIVE IF SOUTHERN HEMI **
C
$INCLUDE:'WRDSIZ.INC'
C
C    MODHIST:
C         1991      FROM NOAA REPORT
C         2003      FIX SOUTH HEMISPHERE NORTHING, CLOSE W-BOUNDARY
C
C -----------------------------------------------------------------
C
C        COMPUTE ZONE AND LONGITUDE OF CENTRAL MERIDIAN
C        ( CM = 3,6,... OR -3,-6 ... )
C
      FLONI  = FLON
      IF ( FLONI.GT. 180.0D0 ) FLONI = FLONI - 360.0D0
      IF ( FLONI.LT.-180.0D0 ) FLONI = FLONI + 360.0D0
C
      IZ   =  ( 31.0D0 + FLONI/6.0D0 + 1.0D-10  )
      IF (IZ .GT. 60 ) IZ = IZ - 60
C
      LONCEM  =  6*IZ  +   177
C
C          BIAS POINTS  LONGITUDE = LONCEM
C                       LATITUDE  = 0.0D0
C
C          FALSE NORTH/EAST
C          BY CONVENTION:   EAST    =   500,000
C                           NORTH   =         0  IN NORTHERN HEMISPHERE
C                                    10,000,000  IN SOUTHERN HEMI.
C
      FLATZ   =  0.0D0
      FLONZ   =  LONCEM
C
      FALSEE  =   500000.0D0
      FALSEN  =        0.0D0
C
C          FIXED SCALE 0.9996 ON CENTRAL MERIDIAN
C          ( DIFFERS OFF THIS -- AND THAT IS WHAT IS RETURNED )
C
      SFCM  =  0.9996D0
C
C           CALL THE GENERAL TRANSVERSE MERCATOR ROUTINE
C
      CALL GD2TM (FLATZ, FLONZ, FALSEN, FALSEE, SFCM, 
     $            FLAT,FLON,  X,YA, SF )
C
      IF ( FLAT.GE.0.0D0 ) THEN
           Y  = YA
      ELSE
           Y  = YA  +  10 000 000.0D0 
           IZ = -IZ
      ENDIF
C
      IZONE  = IZ
C
900   CONTINUE
C
      RETURN
      END
$PAGE
$SUBTITLE:'UTM TO GEODETIC'
      SUBROUTINE UTM2GD ( X,Y,IZONE, FLAT,FLON,SF )
C
C           CONVERT  UNIVERSAL TRANVERSE MERCATOR TO LAT LON
C
C           ( DOES'NT DO STEROGRAPHIC FOR POLAR REGIONS )
C            USES GENERAL TRANSVERS MERCATOR OF NOAS REPORT
C            LISTED BELOW
C
C      INPUT :
C              X        R*8      X OR EASTING (LONGITUDE LIKE) 
C              Y        R*8      Y OR NORTHING ( BOTH IN M )
C              IZONE    I*4      ZONE NUMBER ( 1-60 )
C                                NEGATIVE IN SOUTH OF EQUATOR !!
C      INPUT:
C              FLAT     R*8      LATITUDE IN DEG
C              FLON     R*8      LONGITUDE IN DEG
C              SF       R*8      SCALE AT POINT
C
$INCLUDE:'WRDSIZ.INC'
C
C -----------------------------------------------------------------
C
C        COMPUTE LONGITUDE OF CENTRAL MERIDIAN FROM ZONE
C        ( CM = 3,6,... OR -3,-6 ... )
C
C
      IZ      =  IABS(IZONE)
      LONCEM  =  6*IZ  +  177
C
C          BIAS POINTS  LONGITUDE = LONCEM
C                       LATITUDE  = 0.0D0
C
C          FALSE NORTH/EAST
C          BY CONVENTION:   EAST    =   500,000
C                           NORTH   =         0  IN NORTHERN HEMISPHERE
C                                    10,000,000  IN SOUTHERN HEMI.
C
      FLATZ   =  0.0D0
      FLONZ   =  LONCEM
C
      FALSEE  =   500000.0D0
      FALSEN  =        0.0D0
C
      YA      =  Y
      IF ( IZONE.GE.0 ) THEN  
             FSIGN =  1.0D0
      ELSE
             FSIGN = -1.0D0
             YA    =  10 000 000  - Y
      ENDIF
C
C
C          FIXED SCALE 0.9996 ON CENTRAL MERIDIAN
C          ( DIFFERS OFF THIS -- AND THAT IS WHAT IS RETURNED )
C
      SFCM  =  0.9996D0
C
C           CALL THE GENERAL TRANSVERSE MERCATOR ROUTINE
C
      CALL TM2GD (FLATZ, FLONZ, FALSEN, FALSEE, SFCM, 
     $            X,YA,  FLATA,FLON, SF )
C
      FLAT  = FSIGN * FLATA
C
900   CONTINUE
C
      RETURN
      END
$PAGE
$SUBTITLE:'INITIALIZATION'
      SUBROUTINE  UTMINT
C
C        INITIALIZE CONSTANTS INTO THE UTM COMMON
C        CALL CONSET FOR WORLD CONSTANTS IF NOT SET YET
C
C    ----- NOTES:  
C                   THIS ROUTINE CALLED BY OTHER UTM ROUTINE
C                   IF NOT DONE EXTERNALLY
C
C
$INCLUDE:'WRDSIZ.INC'
$INCLUDE:'UTMCOM.INC'
$INCLUDE:'CONSET.INC'
C
C ---------------------------------------------------
C
      TEST = DABS ( PI - 3.1415927D0 )
      IF ( TEST.GT. 1.0D-5 ) CALL CONSET
C
C
      PIUTM   = PI
C
      AMAJ    = 1000.0D0 * RE
      FLATEN  = FETH
C
      ESQ     = (2.0D0 - FLATEN ) * FLATEN
      E2SQ    = ESQ /  ( 1.0D0 - ESQ )
      ECC3    = FLATEN / ( 2.0D0 - FLATEN )
      E3SQ    = ECC3 ** 2
C
      RREC    = AMAJ*( 1.0D0 - ECC3 ) * ( 1.0D0 - E3SQ ) *
     $               ( 1.0D0 + 2.25D0*E3SQ + 225.D0*E3SQ**2/64.D0)
C
C         COMPUTE CONSTANTS -- ONLY ONES USED 
C
      A1      = 0.0D0
      A2      = ECC3 * ( -1.5D0 + 9.D0*E3SQ / 16.D0 )
      A3      = 0.0D0
      A4      = (15.0D0 * E3SQ / 32.D0) * ( 2.0D0 - E3SQ )
      A5      = 0.0D0
      A6      = -35.0D0 * E3SQ * ECC3 / 48.0D0
      A7      = 0.0D0
      A8      = 315.0D0 * E3SQ**2 / 512.0D0
C
      B0      = 2.0D0 *(A2 - 2.0D0*A4 + 3.0D0*A6 - 4.0D0*A8 )
      B1      = 0.0D0
      B2      = 8.0D0 *(A4 - 4.0D0*A6 + 10.0D0*A8 )
      B3      = 0.0D0
      B4      = 32.0D0 * ( A6 - 6.0D0*A8 )
      B5      = 0.0D0
      B6      = 128.0D0 * A8 
C
      C1     = 0.0D0
      C2     = 1.5D0 * ECC3 * ( 1.0D0 - 9.0D0*E3SQ/16.0D0 )
      C3     = 0.0D0
      C4     = 21.0D0 * E3SQ / 16.0D0  - 55.0D0 * E3SQ**2 / 32.0D0
      C5     = 0.0D0
      C6     = 151.0D0 *ECC3 * E3SQ / 96.0D0
      C7     = 0.0D0
      C8     = 1097.0D0 * E3SQ**2 / 512.0D0
C
      D0     = 2.0D0*( C2 - 2.0D0*C4 + 3.0D0*C6 - 4.0D0*C8 )
      D1     = 0.0D0
      D2     = 8.0D0 * ( C4 - 4.0D0*C6 + 10.0D0*C8 )
      D3     = 0.0D0
      D4     = 32.0D0 * ( C6 - 6.0D0*C8 )
      D5     = 0.0D0
      D6     = 128.0D0 * C8
C
C            ----------------------------------------
C
      RETURN
      END
$PAGE
$SUBTITLE:'WRITE UTM FILE'
      SUBROUTINE WRTUTM (FNAME, NSTA, STATAB , NAMTAB )
C
C     WRITE  THE STATION TABLE -- IN UTM   FORMAT TO RELOAD
C
C      INPUT
C            FNAME            =  FILENAME 
C            NSTA             =  NUMBER OF ITEMS IN TABLE
C            STATAB(7,NSTA)   = STATION TABLE
C            NAMTAB(NSTA)     = NAME TABLE ( C*30)
C
C           STATAB =  TABLE  DIMENSIONED ( 7, MXNSTA )
C             (1,.)   STATION NUMBER
C             (2,.)   GEODETIC LATITUDE (DEG)
C             (3,.)   LONGITUDE ( DEG )
C             (4,.)   ALTITUDE (M)
C             (5,.)   X  (KM)
C             (6,.)   Y  (KM)
C             (7,.)   Z  (KM)
C           NAMTAB  =   ARRAY OF CHARACTER*30 MXNSTA LONG
C
C
$INCLUDE:'WRDSIZ.INC'
C
C
      DIMENSION    STATAB ( 7, NSTA )
      CHARACTER*30 NAMTAB( NSTA)
      CHARACTER*30 FNAME
      CHARACTER*30 NAME
C
C
      CHARACTER*30  FNI
      CHARACTER*5   FNIB
      EQUIVALENCE   ( FNI, FNIB )
C
C ----------------------------------------------------------
C
      LUN = 9
      CLOSE ( LUN )
      OPEN ( LUN, FILE=FNAME, STATUS='UNKNOWN',IOSTAT=IOCK)
      IF ( IOCK.NE. 0 )  GOTO 900
C
      FNI  = FNAME
      IDISK  = 1
      IF ( FNIB.EQ.'CON  ' ) IDISK = 0
      IF ( FNIB.EQ.'CON: ' ) IDISK = 0
      IF ( FNIB.EQ.'LPT1 ' ) IDISK = 0
      IF ( FNIB.EQ.'LPT1:' ) IDISK = 0
      IF ( FNIB.EQ.'LPT2 ' ) IDISK = 0
      IF ( FNIB.EQ.'LPT2:' ) IDISK = 0
C
C
      IF ( IDISK.EQ.0 ) WRITE ( LUN, 2001 ) NSTA
2001  FORMAT(1X, ';',I5,3X,'ITEMS IN UTM STATION TABLE' /
     $       1X, '; NUM ZONE ',
     $           '     X-UTM (M)      Y-UTM (M)       ALT (M) ',
     $           ' NAME' )
C
C
C
      IF ( IDISK.NE.0 ) WRITE ( LUN, 2011 ) NSTA
2011  FORMAT( ';',I5,3X,'ITEMS IN UTM STATION TABLE' /
     $           '; NUM ZONE ',
     $           '     X-UTM (M)      Y-UTM (M)       ALT (M) ',
     $           ' NAME' )
C
C
      DO 100 I = 1, NSTA
C
      ID  = STATAB(1,I)
      GLAT= STATAB(2,I)
      GLON= STATAB(3,I)
      GALT= STATAB(4,I)
      X   = STATAB(5,I)
      Y   = STATAB(6,I)
      Z   = STATAB(7,I)
C
      NAME= NAMTAB(I)
C
      CALL GD2UTM ( GLAT,GLON, XUTM,YUTM, SF, IZONE )
C
C
      IF ( IDISK.EQ.0 ) WRITE(LUN,2002) ID,IZONE,XUTM,YUTM,GALT,NAME
2002  FORMAT( 1X,I5,I5,2F15.3,F15.3,1X,A28 )
C
C
      IF ( IDISK.NE.0 ) WRITE(LUN,2012) ID,IZONE,XUTM,YUTM,GALT,NAME
2012  FORMAT( I5,I5,2F15.3,F15.3,1X,A28 )
C
C
100   CONTINUE
C
900   CONTINUE
C
      CLOSE ( LUN )
C
      RETURN
      END
$PAGE
$SUBTITLE:'GENERAL GEODETIC TO TRANSVERSE MERCATOR'
       SUBROUTINE GD2TM ( FLATZ,FLONCM,FALSEN,FALSEE,SFCM,FLAT,FLON,
     $                    X,Y,SF )
C
C             GEODETIC TO TRANSVERSE MERCATOR 
C             GENERAL VERSION
C
C       SOURCE OF BASIC ROUTINES
C
C                R. Floyd NOAA TECH Rep. NOA 114, 1985
C                Coordinat Conversion for Hydorgaphic Surveying
C
C      ENTERED/ MODIFIED TO SPIT UP INIT. 6-91  JRCLYNCH
C
C
C      INPUT :
C              FLATZ      R*8      FALSE ZERO OF LATITUDE (DEG)
C              FLOMCM     R*8      LONGITUDE OF CENTRAL MERIDIAN (DEG)
C
C              FALSEN     R*8      FALSE NORTHING (M)
C              FALSEE     R*8      FALSE EASTING (M)
C 
C              SFCM       R*8      SCALE FACTOR OF CENTER MERIDIAN
C
C              FLAT       R*8      LATITUDE OF POINT (DEG) 
C              FLON       R*8      LONGITUDE OF POINT (DEG)
C
C    OUTPUT:
C               X         R*8      EASTING (M)
C               Y         R*8      NORTHING (M)
C               SF        R*8      SCALE FACTOR AT POINT
C
$INCLUDE:'WRDSIZ.INC'
$INCLUDE:'UTMCOM.INC'
C
C----------------------------------------------------------------------
C
C             CHECK IF INITIALZIZED YET
C
       TEST = DABS ( PIUTM - 3.1415927D0 ) 
       IF ( TEST.GT.1.0D-5 )  CALL UTMINT
C
C         ------------------------------------------
C
      PI   = PIUTM    
      DTR  = PI / 180.0D0
C
      PHIFAL  =  DTR * FLATZ
      RLONCM  =  DTR * FLONCM
      RLAT    =  DTR * FLAT
      RLON    =  DTR * FLON
C
      CZ      =  DCOS ( PHIFAL )
      OMEGAF  =  PHIFAL + CZ* DSIN(PHIFAL) *
     $                      (B0 + B2*CZ**2 + B4*CZ**4 + B6*CZ**6 )
C
C        ---------------------------------------
C
      YVALUE =  - SFCM * OMEGAF * RREC
C
      C       =  DCOS ( RLAT )
C
C          TANGENT OF LAT -- WITH CARE FOR PI/2 POINT -- POLES
C
      TEST   =   DABS ( DABS ( RLAT ) - PI/2.0D0  )
C
      IF ( TEST.GT.1.0D-15 ) THEN
         T  = DTAN ( RLAT )
      ELSE
         T  = 0.0D0
      ENDIF
C
      T2    =   T**2
      T4    =   T2**2
C
      ETASQ  =  E2SQ * C**2
C
      DLON   =  FLON - FLONCM
      IF ( DLON.GT. 180.0D0 ) DLON = DLON - 360.0D0
      IF ( DLON.LT.-180.0D0 ) DLON = DLON + 360.0D0
      DLON  = DTR * DLON
C
      FL    =  C * DLON
C
      SLAT  =  DSIN ( RLAT )
C
      OMEGA = RLAT + C*SLAT*(B0 + B2*C**2 +B4*C**4 + C6*C**6)
C
      S    =   OMEGA * RREC
      RN   =   AMAJ / DSQRT(1.0D0 - ESQ*SLAT**2 )
C
      E3   =  ( 1.0D0 - T2 + ETASQ ) / 6.0D0
      E4   =  ( 5.0D0 - T2 + ETASQ*(9.0D0 + 4.0D0*ETASQ) )/ 12.0D0
C
      E5   =  ( 5.0D0 - 18.D0*T2 + T4 + 
     $          ETASQ*(14.D0 - 58.D0*T2))/ 120.0D0
C
      E6   =  ( 61.0D0 - 58.0*T2 + T4 +
     $          ETASQ*(270.D0 - 330.D0*T2))/360.0D0
C
      E7   =  (61.0D0 - 479.D0*T2 + 179.D0*T4 - T2*T4 )/ 5040.0D0
C
      F2   =  ( 1.0D0 + ETASQ ) / 2.0D0
      F4   =  ( 5.0D0 - 4.0D0*T2 + ETASQ*(9.D0 - 24.D0*T2))/12.0D0
C
      FL2  =  FL**2
C
      XTRUE = SFCM * RN * FL *
     $        ( 1.0D0 + FL2*( E3 + FL2*( E5 + E7*FL2)))
C
      YTRUE = SFCM * ( S + RN * T * FL2 *
     $        ( 1.0D0 + FL2*( E4 + E6*FL2))/ 2.0D0 )
C
      X     =   XTRUE + FALSEE
      Y     =   YTRUE + FALSEN + YVALUE
C
      SF    =   SFCM * ( 1.0D0 + F2*FL2 * ( 1.0D0 + F4*FL2) )
C
900   CONTINUE
C
      RETURN
      END
$PAGE
$SUBTITLE:'GENERAL TRANSVERSE MERCATOR TO GEODETIC'
       SUBROUTINE TM2GD ( FLATZ,FLONCM,FALSEN,FALSEE,SFCM,X,Y,
     $                    FLAT,FLON,SF )
C
C             TRANSVERSE MERCATOR  TO GEODETIC
C             GENERAL VERSION
C
C       SOURCE OF BASIC ROUTINES
C
C                R. Floyd NOAA TECH Rep. NOA 114, 1985
C                Coordinat Conversion for Hydorgaphic Surveying
C
C      ENTERED/ MODIFIED TO SPIT UP INIT. 6-91  JRCLYNCH
C
C
C      INPUT :
C              FLATZ      R*8      FALSE ZERO OF LATITUDE (DEG)
C              FLOMCM     R*8      LONGITUDE OF CENTRAL MERIDIAN (DEG)
C
C              FALSEN     R*8      FALSE NORTHING (M)
C              FALSEE     R*8      FALSE EASTING (M)
C
C               X         R*8      EASTING (M)
C               Y         R*8      NORTHING (M)
C 
C              SFCM       R*8      SCALE FACTOR OF CENTER MERIDIAN
C
C    OUTPUT:
C
C              FLAT       R*8      LATITUDE OF POINT (DEG) 
C              FLON       R*8      LONGITUDE OF POINT (DEG)
C              SF         R*8      SCALE FACTOR AT POINT
C
$INCLUDE:'WRDSIZ.INC'
$INCLUDE:'UTMCOM.INC'
C
C----------------------------------------------------------------------
C
C             CHECK IF INITIALZIZED YET
C
       TEST = DABS ( PIUTM - 3.1415927D0 ) 
       IF ( TEST.GT.1.0D-5 )  CALL UTMINT
C
C         ------------------------------------------
C
      PI   = PIUTM    
      DTR  = PI / 180.0D0
      RTD  = 180.0D0 / PI
C
      PHIFAL  =  DTR * FLATZ
      RLONCM  =  DTR * FLONCM
C
      CZ      =  DCOS ( PHIFAL )
      OMEGAF  =  PHIFAL + CZ* DSIN(PHIFAL) *
     $                      (B0 + B2*CZ**2 + B4*CZ**4 + B6*CZ**6 )
C
C        ---------------------------------------
C
      YVALUE    = - SFCM * OMEGAF * RREC
C
      OMEGA     =  (Y - FALSEN - YVALUE ) / (SFCM * RREC)
      C         =  DCOS ( OMEGA )
C
C             OMEGA NEAR +/-  PI/2 ( AT POLE )
C
      IF ( DABS(C) .LE. 1.0D-10 ) THEN
           FLAT  = RTD * OMEGA
           FLON  = FLONCM
           SF    = SFCM
           GOTO 900
      ENDIF
C
C               NORMAL CASE
C
      FLATP  = OMEGA + DSIN(OMEGA) * C *
     $                 (D0 + D2*C**2 + D4*C**4 + D6*C**6 )
C
      SLATP  =  DSIN ( FLATP )
      CLATP  =  DCOS ( FLATP )
      T      =  DTAN ( FLATP )
C
      RNP    = AMAJ / DSQRT (1.0D0 - ESQ * SLATP**2 )
      ETASQ  = E2SQ * CLATP**2
C
      T2     =  T**2
      T4     =  T2**2
C
      G2     = -T * ( 1.0D0 + ETASQ ) /2.0D0
      G3     = - (1.0D0 + 2.0D0*T2 + ETASQ ) /6.0D0
C
      G4     = - (5.0D0 + 3.0D0*T2 + ETASQ * (1.0D0 - 9.0D0*T2 ) -
     $                    4.0D0*ETASQ**2 ) /12.0D0
C
      G5     =   (5.0D0 + 28.0D0*T2 + 24.0D0*T4 +
     $                   ETASQ*(6.0D0 +  8.0D0*T2) ) / 120.D0
C
      G6     =  (61.0D0 + 90.0D0*T2 + 45.0D0*T4 +
     $           ETASQ*(46.0D0 - 252.0D0*T2 - 90.0D0*T4) )/ 360.0D0
C
      G7     = -(61.0D0 + 662.0D0*T2 + 1320.0D0*T4 +
     $                    720.0D0*T2*T4 )/ 5040.0D0
C
      H2     =  ( 1.0D0 + ETASQ ) / 2.0D0
      H4     =  ( 1.0D0 + 5.0D0 * ETASQ ) / 12.0D0
C
      Q      =  ( X  - FALSEE ) / ( SFCM * RNP )
      Q2     =  Q**2
C
      FL     = Q * (1.0D0 + Q2* ( G3 + Q2 * ( G5 + G7*Q2) ) )
C
      FLATR  = FLATP + G2*Q2*( 1.0D0 + Q2*( G4 + G6*Q2 ) )
      FLOND  = FLONCM + RTD * ( FL /CLATP )
      IF ( FLOND .GT. 180.0D0 ) FLOND = FLOND - 360.0D0
      IF ( FLOND .LT.-180.0D0 ) FLOND = FLOND + 360.0D0
C
      SF    =  SFCM * ( 1.0D0 + H2*Q2* (1.0D0 + H4*Q2) )
      FLAT  =  RTD   * FLATR
      FLON  =          FLOND
C
900   CONTINUE
C
      RETURN
      END
